#include <string.h>

#include "cv.h"
#include "cvu.h"

#define IMAGE ((const cv_vmemp)0x1c00)

#define NUM_PIECES 6

const char *const titles[NUM_PIECES] = {"Duet 1", "Duet 3", "Duet 4", "Duet 7", "Duet 9", "Duet 10"};
extern const uint16_t duet_1_1[], duet_1_2[], duet_3_1[], duet_3_2[], duet_4_1[], duet_4_2[], duet_7_1[], duet_7_2[], duet_9_1[], duet_9_2[], duet_10_1[], duet_10_2[];
const uint16_t *const voices1[NUM_PIECES] = {duet_1_1, duet_3_1, duet_4_1, duet_7_1, duet_9_1, duet_10_1};
const uint16_t *const voices2[NUM_PIECES] = {duet_1_2, duet_3_2, duet_4_2, duet_7_2, duet_9_2, duet_10_2};

struct cvu_music voice1, voice2;

volatile bool change;

void play(void)
{
	if(change)
		return;
	cvu_play_music(&voice1);
	cvu_play_music(&voice2);
}

void main(void)
{
	uintptr_t a;
	uint_fast8_t i;
	struct cv_controller_state cs;

	// Init screen.
	cv_set_image_table(IMAGE);
	cv_set_colors(CV_COLOR_LIGHT_GREEN, CV_COLOR_BLACK);
	a = *(int *)(0x6c);
	a -= 0x30 * 8;
	cvu_memtovmemcpy(0x1800, (void *)(a), 2048);
	cv_set_character_pattern_t(0x1800);
	cv_set_screen_mode(CV_SCREENMODE_TEXT);
	cvu_vmemset(IMAGE, ' ', 960);
	cvu_memtovmemcpy(IMAGE + 1, "Mozart's horn duets:", 20);
	for(i = 0; i < NUM_PIECES; i++)
	{
		cvu_voutb('1' + i, IMAGE + 120 + 2 * 40 * i + 1);
		cvu_voutb(':', IMAGE + 120 + 2 * 40 * i + 2);
		cvu_memtovmemcpy(IMAGE + 120 + 2 * 40 * i + 4, titles[i], strlen(titles[i]));
	}
	cv_set_screen_active(true);

	// Init music.
	cvu_init_music(&voice1);
	cvu_init_music(&voice2);

	cv_set_vint_handler(&play);

	for(;;)
	{
		do
			cv_get_controller_state(&cs, 0);
		while(cs.keypad < 1 || cs.keypad > NUM_PIECES);

		change = true;
		cvu_init_music(&voice1);
		cvu_init_music(&voice2);
		voice1.channel = CV_SOUNDCHANNEL_0;
		voice2.channel = CV_SOUNDCHANNEL_1;
		voice1.sixteenth_notes_per_second = 5;
		voice2.sixteenth_notes_per_second = 5;
		voice1.notes = voices1[cs.keypad - 1];
		voice2.notes = voices2[cs.keypad - 1];
		change = false;
	}
}

